@@ -17,7 +17,7 @@ import logging
 from typing import Union, Tuple, List
 
 from scipy.stats import wasserstein_distance
-from scipy.stats import loguniform
+from scipy.stats import loguniform, uniform
 
 
 _MSE_DEFAULT = 1e6  #default value for the mean square error, if error occurs
@@ -243,6 +243,43 @@ def check_user_input(esn, u_train: Union[np.ndarray, torch.Tensor] = None,
 
     esn.toTorch()
 
+#--------------------------------------------------------------------------
+#FH 25.05.2022: added minmax_scaling
+def minmax_scaling(x, x_min=None,x_max=None, dataScaling=1):
+    '''
+    Applies min-max scaling to data x. If x_min,x_max not given, they are compute based on min/max values along time axis (axis 0)
+
+    INPUT:
+        x           - data, shape: (timesteps, modes) 
+        x_min       - min value of data x
+        x_max       - max value of data x 
+        dataScaling - scaling factor
+
+    RETURN:
+        x_scaled - x scaled to interval [-dataScaling, dataScaling]
+    '''
+    arr_type = type(x)
+
+    if x_min is None:
+        if arr_type == np.ndarray:
+            x_min = x.min(axis=0)
+        elif arr_type == torch.Tensor:
+            x_min = torch.amin(x,dim=0)
+
+    if x_max is None:
+        if arr_type == np.ndarray:
+            x_max = x.max(axis=0)
+        elif arr_type == torch.Tensor:
+            x_max = torch.amax(x,dim=0)
+            
+
+    x_scaled = (x - x_min)/(x_max- x_min)
+    x_scaled -= 0.5
+    x_scaled *=2*dataScaling
+
+    return x_scaled
+
+
 ###########################################################################################################
 
 #                            RUNNING AN ESN
@@ -722,7 +759,7 @@ def ReadStudy(filepath: str, study_parameters: Union[list,tuple], nstudy: int =
                 study_dicts.append(study_dict)
             
         config = CreateStudyConfigArray(study_parameters, study_dicts)
-        return np.array(mse_train), np.array(mse_test), np.array(y_pred), config
+        return np.array(mse_train), np.array(mse_test), np.array(mse_val), np.array(y_pred_test), np.array(y_pred_val), config
 
     #read all specified seeds
     else:
@@ -775,7 +812,7 @@ def ReadStudy(filepath: str, study_parameters: Union[list,tuple], nstudy: int =
 
 ###########################################################################################################
 
-def InitRandomSearchStudyOrder(nstudy: int, study_tuple: tuple, HP_range_dict: dict = {}) -> list:
+def InitRandomSearchStudyOrder(nstudy: int, study_tuple: tuple, HP_range_dict: dict = {}, use_log_scale: bool=False) -> list:
     ''' Initializes the HP study parameter settings for a random search. 
         Each of the nstudy settings is given by an entry of the returned list config.
 
@@ -783,7 +820,7 @@ def InitRandomSearchStudyOrder(nstudy: int, study_tuple: tuple, HP_range_dict: d
         nstudy           - number of different reservoir setting that should be studied. 
         study_tuple      - tuple specifying the parameters that are studied
         HP_range_dict    - dictionary containing the interval of each hyperparameter (HP must be specified in study_set) 
-
+        use_log_scale    - whether to use loguniform dist. or uniform (nothe that regressionParameter always uses log scale)
     RETURN: 
         config - array indicating the parameter setting for given study
     '''
@@ -798,12 +835,19 @@ def InitRandomSearchStudyOrder(nstudy: int, study_tuple: tuple, HP_range_dict: d
         setting = []
         iparam = 0
         for param in study_tuple:
-            if param == 'regressionParameters':
-                setting.append([loguniform.rvs(HP_range_dict[param][0],HP_range_dict[param][1],size = 1)[0]])
-            elif param == 'n_reservoir':
-                setting.append(int(loguniform.rvs(HP_range_dict[param][0],HP_range_dict[param][1],size = 1)[0]))
-            else:
+            if param == 'regressionParameter':
                 setting.append(loguniform.rvs(HP_range_dict[param][0],HP_range_dict[param][1],size = 1)[0])
+            elif param == 'n_reservoir':
+                if use_log_scale:
+                    setting.append(int(loguniform.rvs(HP_range_dict[param][0],HP_range_dict[param][1],size = 1)[0]))
+                else:
+                    setting.append(int(uniform.rvs(HP_range_dict[param][0],HP_range_dict[param][1],size = 1)[0]))
+            else:
+                if use_log_scale:
+                    setting.append(loguniform.rvs(HP_range_dict[param][0],HP_range_dict[param][1],size = 1)[0])
+                else:
+                    setting.append(uniform.rvs(HP_range_dict[param][0],HP_range_dict[param][1],size = 1)[0])
+                
             
             iparam +=1
         config.append(setting)
